"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.GenericProvider = undefined;

var _bluebirdLstC;

function _load_bluebirdLstC() {
    return _bluebirdLstC = require("bluebird-lst-c");
}

exports.validateUpdateInfo = validateUpdateInfo;

var _restApiRequest;

function _load_restApiRequest() {
    return _restApiRequest = require("../../src/publish/restApiRequest");
}

var _url;

function _load_url() {
    return _url = _interopRequireWildcard(require("url"));
}

var _path = _interopRequireWildcard(require("path"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

class GenericProvider {
    constructor(configuration) {
        this.configuration = configuration;
        this.baseUrl = (_url || _load_url()).parse(this.configuration.url);
        this.channel = this.configuration.channel || "latest";
    }
    getLatestVersion() {
        var _this = this;

        return (0, (_bluebirdLstC || _load_bluebirdLstC()).coroutine)(function* () {
            let result = null;
            try {
                const pathname = _path.posix.resolve(_this.baseUrl.pathname || "/", `${ _this.channel }.yml`);
                result = yield (0, (_restApiRequest || _load_restApiRequest()).request)({ hostname: _this.baseUrl.hostname, port: _this.baseUrl.port || "443", path: `${ pathname }${ _this.baseUrl.search || "" }` });
            } catch (e) {
                if (e instanceof (_restApiRequest || _load_restApiRequest()).HttpError && e.response.statusCode === 404) {
                    throw new Error(`Cannot find channel "${ _this.channel }" update info: ${ e.stack || e.message }`);
                }
                throw e;
            }
            validateUpdateInfo(result);
            return result;
        })();
    }
    getUpdateFile(versionInfo) {
        var _this2 = this;

        return (0, (_bluebirdLstC || _load_bluebirdLstC()).coroutine)(function* () {
            return {
                name: _path.posix.basename(versionInfo.path),
                url: (_url || _load_url()).format(Object.assign({}, _this2.baseUrl, { pathname: _path.posix.resolve(_this2.baseUrl.pathname || "/", versionInfo.path) })),
                sha2: versionInfo.sha2
            };
        })();
    }
}
exports.GenericProvider = GenericProvider;
function validateUpdateInfo(info) {
    if (info.sha2 == null) {
        throw new Error("Update info doesn't contain sha2 checksum");
    }
    if (info.path == null) {
        throw new Error("Update info doesn't contain file path");
    }
}
//# sourceMappingURL=GenericProvider.js.map