"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.GitHubProvider = undefined;

var _bluebirdLstC;

function _load_bluebirdLstC() {
    return _bluebirdLstC = require("bluebird-lst-c");
}

var _restApiRequest;

function _load_restApiRequest() {
    return _restApiRequest = require("../../src/publish/restApiRequest");
}

var _GenericProvider;

function _load_GenericProvider() {
    return _GenericProvider = require("./GenericProvider");
}

var _path = _interopRequireWildcard(require("path"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

class GitHubProvider {
    constructor(options) {
        this.options = options;
    }
    getLatestVersion() {
        var _this = this;

        return (0, (_bluebirdLstC || _load_bluebirdLstC()).coroutine)(function* () {
            // do not use API to avoid limit
            const basePath = _this.getBasePath();
            let version = (yield (0, (_restApiRequest || _load_restApiRequest()).request)({ hostname: "github.com", path: `${ basePath }/latest` })).location;
            const versionPosition = version.lastIndexOf("/") + 1;
            try {
                version = version.substring(version[versionPosition] === "v" ? versionPosition + 1 : versionPosition);
            } catch (e) {
                throw new Error(`Cannot parse extract version from location "${ version }": ${ e.stack || e.message }`);
            }
            let result = null;
            try {
                result = yield (0, (_restApiRequest || _load_restApiRequest()).request)({ hostname: "github.com", path: `https://github.com${ basePath }/download/v${ version }/latest.yml` });
            } catch (e) {
                if (e instanceof (_restApiRequest || _load_restApiRequest()).HttpError && e.response.statusCode === 404) {
                    throw new Error(`Cannot find latest.yml in the latest release artifacts: ${ e.stack || e.message }`);
                }
                throw e;
            }
            (0, (_GenericProvider || _load_GenericProvider()).validateUpdateInfo)(result);
            return result;
        })();
    }
    getBasePath() {
        return `/${ this.options.owner }/${ this.options.repo }/releases`;
    }
    getUpdateFile(versionInfo) {
        var _this2 = this;

        return (0, (_bluebirdLstC || _load_bluebirdLstC()).coroutine)(function* () {
            const basePath = _this2.getBasePath();
            // space is not supported on GitHub
            const name = _path.posix.basename(versionInfo.path).replace(/ /g, "-");
            return {
                name: name,
                url: `https://github.com${ basePath }/download/v${ versionInfo.version }/${ name }`,
                sha2: versionInfo.sha2
            };
        })();
    }
}
exports.GitHubProvider = GitHubProvider; //# sourceMappingURL=GitHubProvider.js.map