"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.NsisUpdater = undefined;

var _bluebirdLstC;

function _load_bluebirdLstC() {
    return _bluebirdLstC = require("bluebird-lst-c");
}

var _bluebirdLstC2;

function _load_bluebirdLstC2() {
    return _bluebirdLstC2 = _interopRequireDefault(require("bluebird-lst-c"));
}

var _events;

function _load_events() {
    return _events = require("events");
}

var _child_process;

function _load_child_process() {
    return _child_process = require("child_process");
}

var _path = _interopRequireWildcard(require("path"));

var _os;

function _load_os() {
    return _os = require("os");
}

var _semver;

function _load_semver() {
    return _semver = require("semver");
}

var _httpRequest;

function _load_httpRequest() {
    return _httpRequest = require("../../src/util/httpRequest");
}

var _BintrayProvider;

function _load_BintrayProvider() {
    return _BintrayProvider = require("./BintrayProvider");
}

var _fsExtraP;

function _load_fsExtraP() {
    return _fsExtraP = require("fs-extra-p");
}

var _jsYaml;

function _load_jsYaml() {
    return _jsYaml = require("js-yaml");
}

var _GenericProvider;

function _load_GenericProvider() {
    return _GenericProvider = require("./GenericProvider");
}

var _GitHubProvider;

function _load_GitHubProvider() {
    return _GitHubProvider = require("./GitHubProvider");
}

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class NsisUpdater extends (_events || _load_events()).EventEmitter {
    constructor(options) {
        super();
        this.updateAvailable = false;
        this.quitAndInstallCalled = false;
        this.quitHandlerAdded = false;
        this.app = global.__test_app || require("electron").app;
        if (options == null) {
            this.clientPromise = this.loadUpdateConfig();
        } else {
            this.setFeedURL(options);
        }
    }
    getFeedURL() {
        return JSON.stringify(this.clientPromise, null, 2);
    }
    setFeedURL(value) {
        this.clientPromise = (_bluebirdLstC2 || _load_bluebirdLstC2()).default.resolve(createClient(value));
    }
    checkForUpdates() {
        var _this = this;

        return (0, (_bluebirdLstC || _load_bluebirdLstC()).coroutine)(function* () {
            if (_this.clientPromise == null) {
                const message = "Update URL is not set";
                const error = new Error(message);
                _this.emit("error", error, message);
                throw error;
            }
            _this.emit("checking-for-update");
            try {
                return yield _this.doCheckForUpdates();
            } catch (e) {
                _this.emit("error", e, (e.stack || e).toString());
                throw e;
            }
        })();
    }
    doCheckForUpdates() {
        var _this2 = this;

        return (0, (_bluebirdLstC || _load_bluebirdLstC()).coroutine)(function* () {
            const client = yield _this2.clientPromise;
            const versionInfo = yield client.getLatestVersion();
            const latestVersion = (0, (_semver || _load_semver()).valid)(versionInfo.version);
            if (latestVersion == null) {
                throw new Error(`Latest version (from update server) is not valid semver version: "${ latestVersion }`);
            }
            const currentVersion = (0, (_semver || _load_semver()).valid)(_this2.app.getVersion());
            if (currentVersion == null) {
                throw new Error(`App version is not valid semver version: "${ currentVersion }`);
            }
            if (!(0, (_semver || _load_semver()).gt)(latestVersion, currentVersion)) {
                _this2.updateAvailable = false;
                _this2.emit("update-not-available");
                return {
                    versionInfo: versionInfo
                };
            }
            const fileInfo = yield client.getUpdateFile(versionInfo);
            _this2.updateAvailable = true;
            _this2.emit("update-available");
            const mkdtemp = require("fs-extra-p").mkdtemp;
            return {
                versionInfo: versionInfo,
                fileInfo: fileInfo,
                downloadPromise: mkdtemp(`${ _path.join((0, (_os || _load_os()).tmpdir)(), "up") }-`).then(function (it) {
                    return (0, (_httpRequest || _load_httpRequest()).download)(fileInfo.url, _path.join(it, fileInfo.name), fileInfo.sha2 == null ? null : { sha2: fileInfo.sha2 });
                }).then(function (it) {
                    _this2.setupPath = it;
                    _this2.addQuitHandler();
                    _this2.emit("update-downloaded", {}, null, versionInfo.version, null, null, function () {
                        _this2.quitAndInstall();
                    });
                    return it;
                }).catch(function (e) {
                    _this2.emit("error", e, (e.stack || e).toString());
                    throw e;
                })
            };
        })();
    }
    addQuitHandler() {
        if (this.quitHandlerAdded) {
            return;
        }
        this.quitHandlerAdded = true;
        this.app.on("quit", () => {
            this.install(true);
        });
    }
    quitAndInstall() {
        if (this.install(false)) {
            this.app.quit();
        }
    }
    install(isSilent) {
        if (this.quitAndInstallCalled) {
            return false;
        }
        const setupPath = this.setupPath;
        if (!this.updateAvailable || setupPath == null) {
            const message = "No update available, can't quit and install";
            this.emit("error", new Error(message), message);
            return false;
        }
        // prevent calling several times
        this.quitAndInstallCalled = true;
        (0, (_child_process || _load_child_process()).spawn)(setupPath, isSilent ? ["/S"] : [], {
            detached: true,
            stdio: "ignore"
        }).unref();
        return true;
    }
    loadUpdateConfig() {
        var _this3 = this;

        return (0, (_bluebirdLstC || _load_bluebirdLstC()).coroutine)(function* () {
            try {
                return createClient((0, (_jsYaml || _load_jsYaml()).safeLoad)((yield (0, (_fsExtraP || _load_fsExtraP()).readFile)(_path.join(global.__test_resourcesPath || process.resourcesPath, "app-update.yml"), "utf-8"))));
            } catch (e) {
                _this3.emit("error", e, (e.stack || e).toString());
                throw e;
            }
        })();
    }
}
exports.NsisUpdater = NsisUpdater;
function createClient(data) {
    if (typeof data === "string") {
        throw new Error("Please pass PublishConfiguration object");
    } else {
        const provider = data.provider;
        switch (provider) {
            case "github":
                return new (_GitHubProvider || _load_GitHubProvider()).GitHubProvider(data);
            case "generic":
                return new (_GenericProvider || _load_GenericProvider()).GenericProvider(data);
            case "bintray":
                return new (_BintrayProvider || _load_BintrayProvider()).BintrayProvider(data);
            default:
                throw new Error(`Unsupported provider: ${ provider }`);
        }
    }
}
//# sourceMappingURL=NsisUpdater.js.map